import * as Sentry from "@sentry/node"
import {
  GraphQLArgumentConfig,
  GraphQLBoolean,
  GraphQLEnumType,
  GraphQLFieldConfig,
  GraphQLFloat,
  GraphQLList,
  GraphQLNonNull,
  GraphQLObjectType,
  GraphQLString,
} from "graphql"
import { connectionFromArraySlice } from "graphql-relay"
import { convertConnectionArgsToGravityArgs } from "lib/helpers"
import {
  isCurrencySupported,
  priceDisplayText,
  priceRangeDisplayText,
} from "lib/moneyHelpers"
import { isNil, map, merge, omit } from "lodash"
import moment from "moment-timezone"
import { pageable } from "relay-cursor-paging"
import {
  connectionWithCursorInfo,
  createPageCursors,
} from "schema/v2/fields/pagination"
import Image, { normalizeImageData } from "schema/v2/image"
import { ResolverContext } from "types/graphql"
import { AuctionResultsAggregationType } from "./aggregations/filterAuctionResultsAggregation"
import { ArtistType } from "./artist"
import date from "./fields/date"
import { InternalIDFields, NodeInterface } from "./object_identification"
// Taken from https://github.com/RubyMoney/money/blob/master/config/currency_iso.json
import { YearRange } from "./types/yearRange"
import { GraphQLError } from "graphql"

export const AuctionResultSorts = {
  type: new GraphQLEnumType({
    name: "AuctionResultSorts",
    values: {
      DATE_DESC: {
        value: "-sale_date",
      },
      DATE_ASC: {
        value: "sale_date",
      },
      PRICE_AND_DATE_DESC: {
        value: "-price_realized_cents_usd,-sale_date",
      },
      ESTIMATE_AND_DATE_DESC: {
        value: "-high_estimate_cents_usd,-sale_date",
      },
    },
  }),
}

export const AuctionResultsState: GraphQLArgumentConfig = {
  type: new GraphQLEnumType({
    name: "AuctionResultsState",
    values: {
      ALL: {
        value: "all",
      },
      PAST: {
        value: "past",
      },
      UPCOMING: {
        value: "upcoming",
      },
    },
  }),
  defaultValue: "all",
  description:
    "State of the returned auction results (can be past, upcoming, or all)",
}

const AuctionResultType = new GraphQLObjectType<any, ResolverContext>({
  name: "AuctionResult",
  interfaces: [NodeInterface],
  fields: () => ({
    ...InternalIDFields,
    slug: {
      type: GraphQLString,
    },
    title: {
      type: GraphQLString,
    },
    artistID: {
      type: new GraphQLNonNull(GraphQLString),
      resolve: ({ artist_id }) => artist_id,
    },
    artist: {
      type: ArtistType,
      description:
        "Be careful when querying for artist data within a connection as it can lead to performance issues.",
      resolve: async (res, _, { artistLoader }) => {
        // In case we get artist already resolved from the main connection
        if (res.artist) {
          return res.artist
        }

        const artist = await artistLoader(res.artist_id)
        return artist
      },
    },
    date,
    dateText: {
      type: GraphQLString,
      resolve: ({ date_text }) => date_text,
    },
    mediumText: {
      type: GraphQLString,
      resolve: ({ medium_text }) => medium_text,
    },
    categoryText: {
      type: GraphQLString,
      resolve: ({ category_text }) => category_text,
    },
    isUpcoming: {
      type: GraphQLBoolean,
      resolve: ({ sale_date }) => {
        if (!sale_date) return null

        return moment(sale_date).isAfter(moment())
      },
    },
    lotNumber: {
      type: GraphQLString,
      resolve: ({ lot_number }) => lot_number,
    },
    comparableAuctionResults: {
      type: auctionResultConnection.connectionType,
      description: "Comparable auction results ",
      args: pageable({}),
      resolve: async (
        parent,
        options,
        { auctionResultComparableAuctionResultsLoader }
      ) => {
        if (!auctionResultComparableAuctionResultsLoader) {
          return null
        }

        const { page, size, offset } = convertConnectionArgsToGravityArgs(
          options
        )

        const response = await auctionResultComparableAuctionResultsLoader(
          parent.id
        )

        if (response.error) {
          return null
        }

        const {
          _embedded: { items },
          total_count,
        } = response

        return merge(
          {
            pageCursors: createPageCursors(
              {
                page,
                size,
              },
              total_count
            ),
          },
          {
            totalCount: total_count,
          },
          connectionFromArraySlice(items, options, {
            arrayLength: total_count,
            sliceStart: offset,
          })
        )
      },
    },
    dimensionText: {
      type: GraphQLString,
      resolve: ({ dimension_text }) => dimension_text,
    },
    dimensions: {
      type: new GraphQLObjectType<any, ResolverContext>({
        name: "AuctionLotDimensions",
        description: "In centimeters.",
        fields: {
          width: {
            type: GraphQLFloat,
          },
          height: {
            type: GraphQLFloat,
          },
          depth: {
            type: GraphQLFloat,
          },
        },
      }),
      resolve: ({ width_cm, height_cm, depth_cm }) => {
        return {
          width: width_cm,
          height: height_cm,
          depth: depth_cm,
        }
      },
    },
    organization: {
      type: GraphQLString,
    },
    saleDate: date,
    saleDateText: {
      type: GraphQLString,
      resolve: ({ sale_date_text }) => sale_date_text,
    },
    saleTitle: {
      type: GraphQLString,
      resolve: ({ sale_title }) => sale_title,
    },
    currency: {
      type: GraphQLString,
    },
    description: {
      type: GraphQLString,
    },
    externalURL: {
      type: GraphQLString,
      resolve: ({ external_url }) => external_url,
    },
    boughtIn: {
      type: GraphQLBoolean,
      resolve: ({ bought_in }) => bought_in,
    },
    location: {
      type: GraphQLString,
    },
    images: {
      type: new GraphQLObjectType<any, ResolverContext>({
        name: "AuctionLotImages",
        fields: {
          larger: {
            type: Image.type,
          },
          thumbnail: {
            type: Image.type,
          },
        },
      }),
      resolve: ({ images }) => {
        if (!images || images.length < 1) {
          return null
        }
        return {
          larger: normalizeImageData(images[0].larger),
          thumbnail: normalizeImageData(images[0].thumbnail),
        }
      },
    },
    isInArtsyAuction: {
      type: new GraphQLNonNull(GraphQLBoolean),
      resolve: ({ organization }) => {
        if (organization === "Artsy Auction") {
          return true
        }
        return false
      },
    },
    performance: {
      type: new GraphQLObjectType({
        name: "AuctionLotPerformance",
        fields: {
          mid: {
            type: GraphQLString,
            description: "Percentage performance over mid-estimate",
            resolve: ({
              price_realized_cents,
              high_estimate_cents,
              low_estimate_cents,
            }) => {
              if (
                price_realized_cents &&
                high_estimate_cents &&
                low_estimate_cents
              ) {
                const midEstimate =
                  (low_estimate_cents + high_estimate_cents) / 2
                const delta = price_realized_cents - midEstimate
                return Math.round((delta / midEstimate) * 100) + "%"
              }
              return null
            },
          },
        },
      }),
      resolve: (lot) => lot,
    },
    estimate: {
      type: new GraphQLObjectType<any, ResolverContext>({
        name: "AuctionLotEstimate",
        fields: {
          low: {
            type: GraphQLFloat,
            resolve: ({ low_estimate_cents }) => low_estimate_cents,
          },
          high: {
            type: GraphQLFloat,
            resolve: ({ high_estimate_cents }) => high_estimate_cents,
          },

          display: {
            type: GraphQLString,
            resolve: ({
              currency,
              low_estimate_cents,
              high_estimate_cents,
            }) => {
              if (!low_estimate_cents && !high_estimate_cents) {
                return null
              }

              if (!isCurrencySupported(currency)) {
                Sentry.captureException(`currency not supported ${currency}}`)
                return null
              }

              return priceRangeDisplayText(
                low_estimate_cents,
                high_estimate_cents,
                currency,
                ""
              )
            },
          },
        },
      }),
      resolve: (lot) => lot,
    },
    priceRealized: {
      type: new GraphQLObjectType<any, ResolverContext>({
        name: "AuctionResultPriceRealized",
        fields: {
          cents: {
            type: GraphQLFloat,
            resolve: ({ price_realized_cents }) => price_realized_cents,
          },
          centsUSD: {
            type: GraphQLFloat,
            resolve: ({ price_realized_cents_usd }) => price_realized_cents_usd,
          },
          display: {
            type: GraphQLString,
            args: {
              format: {
                type: GraphQLString,
                description: "Passes in to numeral, such as `'0.00'`",
                defaultValue: "",
              },
            },
            resolve: ({ currency, price_realized_cents }, { format }) => {
              if (isNil(price_realized_cents)) {
                return null
              }

              if (!isCurrencySupported(currency)) {
                Sentry.captureException(`currency not supported ${currency}}`)
                return null
              }

              return priceDisplayText(price_realized_cents, currency, format)
            },
          },
          displayUSD: {
            type: GraphQLString,
            args: {
              format: {
                type: GraphQLString,
                description: "Passes in to numeral, such as `'0.00'`",
                defaultValue: "",
              },
            },
            resolve: ({ price_realized_cents_usd }, { format }) => {
              if (isNil(price_realized_cents_usd)) {
                return null
              }

              return priceDisplayText(price_realized_cents_usd, "USD", format)
            },
          },
        },
      }),
      resolve: (lot) => lot,
    },
  }),
})

export const AuctionResult: GraphQLFieldConfig<void, ResolverContext> = {
  type: AuctionResultType,
  description: "An auction result",
  args: {
    id: {
      description: "The ID or slug of the auction result",
      type: new GraphQLNonNull(GraphQLString),
    },
  },
  resolve: (_root, { id }, { auctionLotLoader }) => {
    if (!auctionLotLoader) {
      return null
    }
    return auctionLotLoader(id)
  },
}

export const auctionResultConnection = connectionWithCursorInfo({
  nodeType: AuctionResultType,
  connectionFields: {
    createdYearRange: {
      resolve: async ({ artist_id }, _, { auctionResultFilterLoader }) => {
        if (!auctionResultFilterLoader) {
          return null
        }

        try {
          const aggregations = await auctionResultFilterLoader({
            artist_id,
            aggregations: ["lots_by_created_year"],
          })

          if (
            aggregations &&
            aggregations["lots_by_created_year"] &&
            Object.keys(aggregations["lots_by_created_year"]).length > 0
          ) {
            const createdYearRange = Object.keys(
              aggregations["lots_by_created_year"]
            )

            if (createdYearRange.length > 0) {
              return {
                startAt: createdYearRange[0],
                endAt: createdYearRange[createdYearRange.length - 1],
              }
            }
          }
          return {
            startAt: null,
            endAt: null,
          }
        } catch (error) {
          throw new GraphQLError(`createdYearRange resolver error: ${error}`)
        }
      },
      type: YearRange,
    },
    aggregations: {
      type: new GraphQLList(AuctionResultsAggregationType),
      resolve: ({ aggregations }) =>
        map(omit(aggregations, "cached"), (counts, slice) => ({
          counts,
          slice,
        })),
    },
  },
})
